"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const dotenv_1 = __importDefault(require("dotenv"));
require("mocha");
const moment_1 = __importDefault(require("moment"));
const utils_general_1 = require("@energyweb/utils-general");
const ethers_1 = require("ethers");
const migrate_1 = require("../migrate");
const __1 = require("..");
describe('Issuer', () => {
    let blockchainProperties;
    dotenv_1.default.config({
        path: '.env.test'
    });
    const [web3Url] = process.env.WEB3.split(';');
    const provider = utils_general_1.getProviderWithFallback(web3Url);
    const deviceOwnerPK = '0x622d56ab7f0e75ac133722cc065260a2792bf30ea3265415fe04f3a2dba7e1ac';
    const deviceOwnerWallet = new ethers_1.Wallet(deviceOwnerPK, provider);
    const issuerPK = '0x50397ee7580b44c966c3975f561efb7b58a54febedaa68a5dc482e52fb696ae7';
    const issuerWallet = new ethers_1.Wallet(issuerPK, provider);
    let timestamp = moment_1.default().subtract(10, 'year').unix();
    const setActiveUser = (wallet) => {
        blockchainProperties.activeUser = wallet;
    };
    const createCertificationRequest = async (fromTime, toTime, deviceId, forAddress) => {
        setActiveUser(deviceOwnerWallet);
        const generationFromTime = fromTime !== null && fromTime !== void 0 ? fromTime : timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const generationToTime = toTime !== null && toTime !== void 0 ? toTime : timestamp;
        const device = deviceId !== null && deviceId !== void 0 ? deviceId : '1';
        const certificationRequest = await __1.CertificationRequest.create(generationFromTime, generationToTime, device, blockchainProperties, forAddress);
        return certificationRequest;
    };
    it('migrates Issuer and Registry', async () => {
        const registry = await migrate_1.migrateRegistry(provider, issuerPK);
        const issuer = await migrate_1.migrateIssuer(provider, issuerPK, registry.address);
        const version = await issuer.version();
        chai_1.assert.equal(version, 'v0.1');
        const registryAddress = await issuer.getRegistryAddress();
        chai_1.assert.equal(registryAddress, registry.address);
        blockchainProperties = {
            web3: provider,
            activeUser: issuerWallet,
            registry,
            issuer
        };
    });
    it('gets all certification requests', async () => {
        await createCertificationRequest();
        await createCertificationRequest();
        const allCertificationRequests = await __1.CertificationRequest.getAll(blockchainProperties);
        chai_1.assert.equal(allCertificationRequests.length, 2);
    });
    it('user correctly requests issuance', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const certificationRequest = await createCertificationRequest(fromTime, toTime, device);
        chai_1.assert.isAbove(certificationRequest.id, -1);
        chai_1.assert.deepOwnInclude(certificationRequest, {
            deviceId: device,
            owner: deviceOwnerWallet.address,
            fromTime,
            toTime,
            approved: false
        });
    });
    it('issuer correctly approves issuance', async () => {
        const volume = ethers_1.BigNumber.from(1e9);
        let certificationRequest = await createCertificationRequest();
        setActiveUser(issuerWallet);
        const certificateId = await certificationRequest.approve(volume);
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        const deviceOwnerBalance = await blockchainProperties.registry.balanceOf(deviceOwnerWallet.address, certificateId);
        chai_1.assert.equal(deviceOwnerBalance.toString(), volume.toString());
    });
    it('user revokes a certificationRequest', async () => {
        setActiveUser(deviceOwnerWallet);
        let certificationRequest = await createCertificationRequest();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isFalse(certificationRequest.approved);
        chai_1.assert.isFalse(certificationRequest.revoked);
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.revoked);
    });
    it('issuer should be able to revoke a certificationRequest', async () => {
        setActiveUser(issuerWallet);
        let certificationRequest = await createCertificationRequest();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isFalse(certificationRequest.revoked);
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.revoked);
    });
    it('user shouldnt be able to revoke an approved certificationRequest', async () => {
        setActiveUser(deviceOwnerWallet);
        const volume = ethers_1.BigNumber.from(1e9);
        let certificationRequest = await createCertificationRequest();
        setActiveUser(issuerWallet);
        certificationRequest = await certificationRequest.sync();
        await certificationRequest.approve(volume);
        setActiveUser(deviceOwnerWallet);
        certificationRequest = await certificationRequest.sync();
        chai_1.assert.isTrue(certificationRequest.approved);
        chai_1.assert.isFalse(certificationRequest.revoked);
        let failed = false;
        try {
            await certificationRequest.revoke();
        }
        catch (e) {
            failed = true;
        }
        chai_1.assert.isTrue(failed);
    });
    // it('should fail to request 2 certificates with the same generation period', async () => {
    //     setActiveUser(deviceOwnerWallet);
    //     const fromTime = timestamp;
    //     // Simulate time moving forward 1 month
    //     timestamp += 30 * 24 * 3600;
    //     const toTime = timestamp;
    //     const device = '1';
    //     await createCertificationRequest(fromTime, toTime, device);
    //     let failed = false;
    //     try {
    //         await createCertificationRequest(fromTime, toTime, device);
    //     } catch (e) {
    //         failed = true;
    //     }
    //     assert.isTrue(failed);
    // });
    it('should request the same certificate after revoking one', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        let certificationRequest = await createCertificationRequest(fromTime, toTime, device);
        certificationRequest = await certificationRequest.sync();
        await certificationRequest.revoke();
        certificationRequest = await certificationRequest.sync();
        const newCertificationRequest = await createCertificationRequest(fromTime, toTime, device);
        chai_1.assert.exists(newCertificationRequest);
    });
    // it('user correctly requests private issuance', async () => {
    //     const volume = BigNumber.from(1e9);
    //     const certificationRequest = await createCertificationRequest(volume);
    //     assert.isAbove(Number(certificationRequest.id), -1);
    //     assert.deepOwnInclude(certificationRequest, {
    //         deviceId: '1',
    //         owner: deviceOwnerWallet.address,
    //         approved: false,
    //         energy: volume
    //     } as Partial<CertificationRequest>);
    // });
    // it('issuer correctly approves private issuance', async () => {
    //     const volume = BigNumber.from(1e9);
    //     let certificationRequest = await createCertificationRequest(volume);
    //     setActiveUser(issuerWallet);
    //     const certificateId = await certificationRequest.approve();
    //     certificationRequest = await certificationRequest.sync();
    //     assert.isTrue(certificationRequest.approved);
    //     const deviceOwnerBalance = await blockchainProperties.registry.balanceOf(
    //         deviceOwnerWallet.address,
    //         Number(certificateId)
    //     );
    //     assert.equal(deviceOwnerBalance.toString(), '0');
    // });
    it('should be able to request for other address', async () => {
        setActiveUser(deviceOwnerWallet);
        const fromTime = timestamp;
        // Simulate time moving forward 1 month
        timestamp += 30 * 24 * 3600;
        const toTime = timestamp;
        const device = '1';
        const certificationRequest = await createCertificationRequest(fromTime, toTime, device, issuerWallet.address);
        chai_1.assert.isOk(certificationRequest);
    });
});
//# sourceMappingURL=Issuer.test.js.map